var $a9I23$fidmx509 = require("@fidm/x509");
var $a9I23$crypto = require("crypto");
var $a9I23$nodeforge = require("node-forge");
var $a9I23$eckey = require("ec-key");


function $parcel$exportWildcard(dest, source) {
  Object.keys(source).forEach(function(key) {
    if (key === 'default' || key === '__esModule' || Object.prototype.hasOwnProperty.call(dest, key)) {
      return;
    }

    Object.defineProperty(dest, key, {
      enumerable: true,
      get: function get() {
        return source[key];
      }
    });
  });

  return dest;
}

function $parcel$export(e, n, v, s) {
  Object.defineProperty(e, n, {get: v, set: s, enumerable: true, configurable: true});
}
var $fbc18d05296a0f5d$exports = {};

$parcel$export($fbc18d05296a0f5d$exports, "EcDecryptionStrategy", () => $fbc18d05296a0f5d$export$14d91d60bcc7352f);




// https://images.apple.com/certificateauthority/pdf/Apple_WWDR_CPS_v1.31.pdf
const $fbc18d05296a0f5d$var$MERCHANT_ID_FIELD_OID = "1.2.840.113635.100.6.32";
class $fbc18d05296a0f5d$export$14d91d60bcc7352f {
    constructor({ certificatePem: certificatePem, privateKeyPem: privateKeyPem }){
        this.privateKey = new $a9I23$eckey(privateKeyPem, "pem");
        this.merchantId = this.extractMerchantId(certificatePem);
    }
    /**
   * Decrypting the token using the PEM formatted merchant certificate and private key (the latter of which, at least, is managed by a third-party)
   */ decrypt(paymentData) {
        const { header: { ephemeralPublicKey: ephemeralPublicKey }, data: data } = paymentData;
        const sharedSecret = this.generateSharedSecret(ephemeralPublicKey);
        const symmetricKey = this.deriveSymmetricKey(sharedSecret);
        const decrypted = this.decryptCiphertext(symmetricKey, data);
        try {
            return JSON.parse(decrypted);
        } catch (error) {
            const err = new Error("Unexpected format of decrypted data. Please check payment processing certificate and its private key.");
            err.stack = error.stack;
            throw err;
        }
    }
    /**
   * Generating the shared secret with the merchant private key and the ephemeral public key(part of the payment token data)
   * using Elliptic Curve Diffie-Hellman (id-ecDH 1.3.132.1.12).
   * As the Apple Pay certificate is issued using prime256v1 encryption, create elliptic curve key instances using the package - https://www.npmjs.com/package/ec-key
   */ generateSharedSecret(ephemeralPublicKey) {
        const prv = new $a9I23$eckey(this.privateKey, "pem"); // Create a new ECkey instance from PEM formatted string
        const publicEc = new $a9I23$eckey(ephemeralPublicKey, "spki"); // Create a new ECKey instance from a base-64 spki string
        return prv.computeSecret(publicEc).toString("hex"); // Compute secret using private key for provided ephemeral public key
    }
    /**
   * Extracting merchant id from merchant certificate
   * Merchant ID is the data of the extension 1.2.840.113635.100.6.32, which is the merchant identifier field (OID 1.2.840.113635.100.6.32).
   * This an id extension of the certificate it’s not your merchant identifier.
   * Parsing the certificate with the x509 NPM package - https://www.npmjs.com/package/x509#x509parsecert-cert-
   */ extractMerchantId(cert) {
        try {
            const info = $a9I23$fidmx509.Certificate.fromPEM(typeof cert === "string" ? Buffer.from(cert) : cert);
            const oid = info.extensions.find((extension)=>extension.oid === $fbc18d05296a0f5d$var$MERCHANT_ID_FIELD_OID);
            return oid.value.toString().slice(2);
        } catch  {
            throw new Error("Unable to extract merchant ID from certificate.");
        }
    }
    /**
   * Derive the symmetric key using the key derivation function described in NIST SP 800-56A, section 5.8.1
   * https://nvlpubs.nist.gov/nistpubs/Legacy/SP/nistspecialpublication800-56ar.pdf
   * The symmetric key is a sha256 hash that contains shared secret token plus encoding information
   */ deriveSymmetricKey(sharedSecret) {
        const KDF_ALGORITHM = "\rid-aes256-GCM"; // The byte (0x0D) followed by the ASCII string "id-aes256-GCM". The first byte of this value is an unsigned integer that indicates the string’s length in bytes; the remaining bytes are a constiable-length string.
        const KDF_PARTY_V = Buffer.from(this.merchantId, "hex").toString("binary"); // The SHA-256 hash of your merchant ID string literal; 32 bytes in size.
        const KDF_PARTY_U = "Apple"; // The ASCII string "Apple". This value is a fixed-length string.
        const KDF_INFO = KDF_ALGORITHM + KDF_PARTY_U + KDF_PARTY_V;
        const hash = $a9I23$crypto.createHash("sha256");
        hash.update(Buffer.from("000000", "hex"));
        hash.update(Buffer.from("01", "hex"));
        hash.update(Buffer.from(sharedSecret, "hex"));
        hash.update(KDF_INFO, "binary");
        return hash.digest("hex");
    }
    /**
   * Decrypting the cipher text from the token (data in the original payment token) key using AES–256 (id-aes256-GCM 2.16.840.1.101.3.4.1.46), with an initialization vector of 16 null bytes and no associated authentication data.
   *
   */ decryptCiphertext(symmetricKey, data) {
        const decoded = $a9I23$nodeforge.util.decode64(data);
        const SYMMETRIC_KEY = $a9I23$nodeforge.util.createBuffer(Buffer.from(symmetricKey, "hex").toString("binary"));
        const IV = $a9I23$nodeforge.util.createBuffer(Buffer.from([
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0
        ]).toString("binary")); // Initialization vector of 16 null bytes
        const CIPHERTEXT = $a9I23$nodeforge.util.createBuffer(decoded.slice(0, -16));
        const decipher = $a9I23$nodeforge.cipher.createDecipher("AES-GCM", SYMMETRIC_KEY); // Creates and returns a Decipher object that uses the given algorithm and password (key)
        const tag = decoded.slice(-16, decoded.length);
        decipher.start({
            iv: IV,
            tagLength: 128,
            tag: $a9I23$nodeforge.util.createBuffer(tag)
        });
        decipher.update(CIPHERTEXT);
        decipher.finish();
        return Buffer.from(decipher.output.toHex(), "hex").toString("utf-8");
    }
}


var $647b0a3116cdbc7c$exports = {};

$parcel$export($647b0a3116cdbc7c$exports, "RsaDecryptionStrategy", () => $647b0a3116cdbc7c$export$196403f4e15ab019);

class $647b0a3116cdbc7c$export$196403f4e15ab019 {
    constructor({ privateKeyPem: privateKeyPem }){
        this.privateKey = privateKeyPem;
    }
    /**
   * Decrypting the token using the PEM formatted merchant certificate and private key (the latter of which, at least, is managed by a third-party)
   */ decrypt(paymentData) {
        const { header: { wrappedKey: wrappedKey }, data: data } = paymentData;
        const restoredKey = this.decryptWrappedKey(wrappedKey);
        const decrypted = this.decryptCiphertext(restoredKey, data);
        try {
            return JSON.parse(decrypted);
        } catch (error) {
            const err = new Error("Unexpected format of decrypted data. Please check payment processing certificate and its private key.");
            err.stack = error.stack;
            throw err;
        }
    }
    /**
   * Decrypting the wrappedKey with the merchant private key using by using the RSA/ECB/OAEPWithSHA256AndMGF1Padding algorithm.
   */ decryptWrappedKey(wrappedKey) {
        const key = $a9I23$nodeforge.pki.privateKeyFromPem(this.privateKey.toString("utf-8"));
        return key.decrypt($a9I23$nodeforge.util.decode64(wrappedKey), "RSA-OAEP", {
            md: $a9I23$nodeforge.md.sha256.create(),
            mgf1: {
                md: $a9I23$nodeforge.md.sha256.create()
            }
        });
    }
    /**
   * Decrypting the cipher text from the token (data in the original payment token) key using AES–128 (id-aes128-GCM 2.16.840.1.101.3.4.1.6), with an initialization vector of 16 null bytes and no associated authentication data.
   *
   */ decryptCiphertext(symmetricKey, data) {
        const decoded = $a9I23$nodeforge.util.decode64(data);
        const SYMMETRIC_KEY = $a9I23$nodeforge.util.createBuffer(symmetricKey);
        const iv = $a9I23$nodeforge.util.createBuffer(Buffer.from([
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0
        ]).toString("binary")); // Initialization vector of 16 null bytes
        const CIPHERTEXT = $a9I23$nodeforge.util.createBuffer(decoded.slice(0, -16));
        const decipher = $a9I23$nodeforge.cipher.createDecipher("AES-GCM", SYMMETRIC_KEY); // Creates and returns a Decipher object that uses the given algorithm and password (key)
        const tag = decoded.slice(-16, decoded.length);
        decipher.start({
            iv: iv,
            tagLength: 128,
            tag: $a9I23$nodeforge.util.createBuffer(tag)
        });
        decipher.update(CIPHERTEXT);
        decipher.finish();
        return Buffer.from(decipher.output.toHex(), "hex").toString("utf-8");
    }
}


var $2a8c82743ff6d561$exports = {};

$parcel$export($2a8c82743ff6d561$exports, "ApplePaymentTokenContext", () => $2a8c82743ff6d561$export$ae64e562fdd1bc5c);
class $b031fed3f543491d$export$26dc4dd0282cdb70 extends Error {
    constructor(message, errors){
        super(message);
        this.errors = errors;
        this.name = "ApplePaymentDecryptionError";
    }
}




class $2a8c82743ff6d561$export$ae64e562fdd1bc5c {
    constructor(options){
        this.options = options;
        if (!options.merchants.length) throw new (0, $b031fed3f543491d$export$26dc4dd0282cdb70)("No merchant configuration provided for decryption context.");
    }
    decrypt(paymentData) {
        const errors = [];
        for (const merchant of this.options.merchants)try {
            return this.decryptForMerchant(paymentData, merchant);
        } catch (error) {
            error.merchantIdentifier = merchant.identifier;
            errors.push(error);
        }
        throw new (0, $b031fed3f543491d$export$26dc4dd0282cdb70)("Failed to decrypt payment data using provided merchant configuration(s).", errors);
    }
    decryptForMerchant(paymentData, merchant) {
        const strategy = paymentData.version === "EC_v1" ? new (0, $fbc18d05296a0f5d$export$14d91d60bcc7352f)(merchant) : new (0, $647b0a3116cdbc7c$export$196403f4e15ab019)(merchant);
        return strategy.decrypt(paymentData);
    }
}


var $374d09f5e1012089$exports = {};
var $f470fc28b1b727c7$exports = {};


var $8a8ed0848bd33910$exports = {};


$parcel$exportWildcard($374d09f5e1012089$exports, $f470fc28b1b727c7$exports);
$parcel$exportWildcard($374d09f5e1012089$exports, $8a8ed0848bd33910$exports);


$parcel$exportWildcard(module.exports, $fbc18d05296a0f5d$exports);
$parcel$exportWildcard(module.exports, $647b0a3116cdbc7c$exports);
$parcel$exportWildcard(module.exports, $2a8c82743ff6d561$exports);
$parcel$exportWildcard(module.exports, $374d09f5e1012089$exports);


